==> pyrlprob-2.3.3.tar.gz: pyrlprob-2.3.3/pyrlprob/tests/py_tests/landing1d.py <==
import numpy as np
from numpy.random import uniform
from typing import *

import gym
from gym import spaces

from pyrlprob.mdp import AbstractMDP

from pyrlprob.tests.py_tests.landing1d_gym import *
from pyrlprob.tests.py_tests.landing1d_dyn import *


class pyLanding1DEnv(AbstractMDP):
    """
    One-Dimensional Landing Problem.
    Reference: https://doi.org/10.2514/6.2008-6615
    """

    def __init__(self, config) -> None:
        """
        Definition of observation and action spaces
        """

        if (not config) or (config is None):
            config = DEFAULT_CONFIG_LANDING1D

        super().__init__(config=config)

        #Time step
        self.time_step = self.tf/(float(self.H))

        #Maximum episode steps
        self.max_episode_steps = self.H

        #Observation space
        self.observation_space = spaces.Box(low=-np.inf, high=np.inf, shape=(4,), dtype=np.float32)
        # # # TEST: image + state
        # # self.observation_space = spaces.Tuple((spaces.Box(low=0, high=255, \
        # #         shape=(256, 256, 1), dtype=np.uint8), \
        # #             spaces.Box(low=-np.inf, high=np.inf, shape=(4,), dtype=np.float32)))
        # # self.observation_space = spaces.Tuple(( spaces.Tuple((spaces.Box(low=0, high=255, \
        # #          shape=(256, 256, 1), dtype=np.uint8), \
        # #              spaces.Box(low=-np.inf, high=np.inf, shape=(4,), dtype=np.float32))), \
        # #                                        spaces.Box(low=-np.inf, high=np.inf, shape=(3,), dtype=np.float32)))

        #Action space
        self.action_space = spaces.Box(low=-1., high=1., shape=(1,), dtype=np.float32)

        #Reward range
        self.reward_range = (-float('inf'), 0.)


    def get_observation(self,
                        state,
                        control) -> np.ndarray:
        """
        Get current observation: height and velocity
        """

        observation = np.array([state["h"], state["v"], state["m"], state["t"]], dtype=np.float32)

        # # # TEST: generate random image here
        # # image = np.random.randint(0, 256, (256, 256, 1), dtype=np.uint8)

        # # observation = (image, np.array([state["h"], state["v"], state["m"], state["t"]], dtype=np.float32))
        # # observation = ((image, np.array([state["h"], state["v"], state["m"], state["t"]], dtype=np.float32)), \
        # #                np.array([state["h"], state["v"], state["m"]], dtype=np.float32))


        return observation
    

    def get_control(self,
                    action,
                    state) -> float:
        """
        Get current control: thrust value
        """

        control = 0.5*(action[0] + 1.)*self.Tmax

        return control


    def next_state(self,
                   state: Optional[Any], 
                   control: Any,
                   time_step: float) -> Dict[str, float]:
        """
        Propagate state: integration of system dynamics
        """

        #State at current time-step
        s = np.array([state["h"], state["v"], state["m"]])

        #Integration of equations of motion
        data = np.array([self.g, control, self.c])

        # Integration
        t_eval = np.array([state['t'], state['t'] + time_step])
        sol = rk4(dynamics, s, t_eval, data)

        #State at next time-step
        self.success = True
        s_new = {"h": sol[0], "v": sol[1], "m": sol[2], "t": t_eval[-1], "step": state["step"] + 1}
        
        return s_new
    

    def collect_reward(self,
                       prev_state: Optional[Any]=None, 
                       state: Optional[Any]=None, 
                       control: Optional[Any]=None) -> Tuple[float, bool]:
        """
        Get current reward and done signal.
        """

        done = False

        reward = state["m"] - prev_state["m"]

        if state["step"] == self.H:
            done = True
        if not self.success:
            done = True
        if state["h"] <= 0. or state["m"] <= 0.:
            done = True
        
        if done:
            cstr_viol = max(abs(state["h"] - self.hf), abs(state["v"] - self.vf) - 0.005)
            state["cstr_viol"] = max(cstr_viol, 0.)

            reward = reward - 10.*cstr_viol
        
        return reward, done
    

    def get_info(self,
                 prev_state,
                 state,
                 observation,
                 control,
                 reward,
                 done) -> Dict[str, float]:
        """
        Get current info.
        """

        info = {}
        info["episode_step_data"] = {}
        info["episode_step_data"]["h"] = [prev_state["h"]] 
        info["episode_step_data"]["v"] = [prev_state["v"]] 
        info["episode_step_data"]["m"] = [prev_state["m"]] 
        info["episode_step_data"]["t"] = [prev_state["t"]] 
        info["episode_step_data"]["T"] = [control]
        if done:
            info["custom_metrics"] = {}
            info["episode_end_data"] = {}

            info["episode_step_data"]["h"].append(state["h"]) 
            info["episode_step_data"]["v"].append(state["v"]) 
            info["episode_step_data"]["m"].append(state["m"]) 
            info["episode_step_data"]["t"].append(state["t"]) 
            info["episode_step_data"]["T"].append(control)

            info["custom_metrics"]["cstr_viol"] = state["cstr_viol"]
            info["episode_end_data"]["hf"] = [state["h"]]
            info["episode_end_data"]["vf"] = [state["v"]]
            info["episode_end_data"]["mf"] = [state["m"]]

        return info
    

    def reset(self) -> np.ndarray:
        """ 
        Reset the environment
        """

        self.state = {}
        self.state["h"] = uniform(self.h0_min, self.h0_max)
        self.state["v"] = uniform(self.v0_min, self.v0_max)
        self.state["m"] = self.m0
        self.state["t"] = 0.
        self.state["step"] = 0

        control = 0.

        observation = self.get_observation(self.state, control)

        return observation

==> pyrlprob-3.2.1.tar.gz: pyrlprob-3.2.1/pyrlprob/tests/py_tests/landing1d.py <==
import numpy as np
from numpy.random import uniform
from typing import *

import gymnasium as gym
from gymnasium import spaces

from pyrlprob.mdp import AbstractMDP

from pyrlprob.tests.py_tests.landing1d_gym import *
from pyrlprob.tests.py_tests.landing1d_dyn import *


class pyLanding1DEnv(AbstractMDP):
    """
    One-Dimensional Landing Problem.
    Reference: https://doi.org/10.2514/6.2008-6615
    """

    def __init__(self, config) -> None:
        """
        Definition of observation and action spaces
        """

        if (not config) or (config is None):
            config = DEFAULT_CONFIG_LANDING1D

        super().__init__(config=config)

        #Time step
        self.time_step = self.tf/(float(self.H))

        #Maximum episode steps
        self.max_episode_steps = self.H

        #Observation space
        self.observation_space = spaces.Box(low=-np.inf, high=np.inf, shape=(4,), dtype=np.float32)

        #Action space
        self.action_space = spaces.Box(low=-1., high=1., shape=(1,), dtype=np.float32)

        #Reward range
        self.reward_range = (-float('inf'), 0.)


    def get_observation(self,
                        state,
                        control) -> np.ndarray:
        """
        Get current observation: height and velocity
        """

        observation = np.array([state["h"], state["v"], state["m"], state["t"]], dtype=np.float32)

        return observation
    

    def get_control(self,
                    action,
                    state) -> float:
        """
        Get current control: thrust value
        """

        control = 0.5*(action[0] + 1.)*self.Tmax

        return control


    def next_state(self,
                   state: Optional[Any], 
                   control: Any,
                   time_step: float) -> Dict[str, float]:
        """
        Propagate state: integration of system dynamics
        """

        #State at current time-step
        s = np.array([state["h"], state["v"], state["m"]])

        #Integration of equations of motion
        data = np.array([self.g, control, self.c])

        # Integration
        t_eval = np.array([state['t'], state['t'] + time_step])
        sol = rk4(dynamics, s, t_eval, data)

        #State at next time-step
        self.success = True
        s_new = {"h": sol[0], "v": sol[1], "m": sol[2], "t": t_eval[-1], "step": state["step"] + 1}
        
        return s_new
    

    def collect_reward(self,
                       prev_state: Optional[Any]=None, 
                       state: Optional[Any]=None, 
                       control: Optional[Any]=None) -> Tuple[float, bool, bool]:
        """
        Get current reward and terminated/truncated signals.
        """

        terminated = False
        truncated = False

        reward = state["m"] - prev_state["m"]

        if state["step"] == self.H:
            terminated = True
        if not self.success:
            truncated = True
        if state["h"] <= 0. or state["m"] <= 0.:
            truncated = True
        
        if terminated or truncated:
            cstr_viol = max(abs(state["h"] - self.hf), abs(state["v"] - self.vf))
            state["cstr_viol"] = max(cstr_viol, 0.)

            reward = reward - 10.*cstr_viol
        
        return reward, terminated, truncated
    

    def get_info(self,
                 prev_state,
                 state,
                 observation,
                 control,
                 reward,
                 done) -> Dict[str, float]:
        """
        Get current info.
        """

        info = {}
        info["episode_step_data"] = {}
        info["episode_step_data"]["h"] = [prev_state["h"]] 
        info["episode_step_data"]["v"] = [prev_state["v"]] 
        info["episode_step_data"]["m"] = [prev_state["m"]] 
        info["episode_step_data"]["t"] = [prev_state["t"]] 
        info["episode_step_data"]["T"] = [control]
        if done:
            info["custom_metrics"] = {}
            info["episode_end_data"] = {}

            info["episode_step_data"]["h"].append(state["h"]) 
            info["episode_step_data"]["v"].append(state["v"]) 
            info["episode_step_data"]["m"].append(state["m"]) 
            info["episode_step_data"]["t"].append(state["t"]) 
            info["episode_step_data"]["T"].append(control)

            info["custom_metrics"]["cstr_viol"] = state["cstr_viol"]
            info["episode_end_data"]["hf"] = [state["h"]]
            info["episode_end_data"]["vf"] = [state["v"]]
            info["episode_end_data"]["mf"] = [state["m"]]

        return info
    

    def reset(self, seed=None, options=None) -> Tuple[np.ndarray, Dict[str, any]]:
        """ 
        Reset the environment
        """

        self.state = {}
        self.state["h"] = uniform(self.h0_min, self.h0_max)
        self.state["v"] = uniform(self.v0_min, self.v0_max)
        self.state["m"] = self.m0
        self.state["t"] = 0.
        self.state["step"] = 0

        control = 0.

        observation = self.get_observation(self.state, control)
        info = self.get_info(self.state, self.state, observation, control, 0., False)

        return observation, info
